// Use this program at your OWN RISK.
// If you want to change it and redistribute, you can, but you can never sell this
// program.
// All the changes to this program must have my name in it.
// Author: Paulo Francisco Zemek
// Homepage: http://www.druids.eti.br/pfz
// e-mails: pzemek@druids.eti.br
//          pzemek@yahoo.com
//          pzemek@bemail.org

#include <Application.h>
#include <Alert.h>
#include <Directory.h>
#include <Entry.h>
#include <fcntl.h>
#include <fs_attr.h>
#include <image.h>
#include <Path.h>
#include <Roster.h>
#include <Screen.h>
#include <stdlib.h>
#include <StringView.h>
#include <termios.h>
#include <Window.h>

#define ShowMessage(msg) ((new BAlert(msg,msg,"OK"))->Go())

class SSProcessRefsLooper:public BLooper{
 public:
  SSProcessRefsLooper():BLooper("Stripper"){}
  virtual void MessageReceived(BMessage *msg);
};

class SSApp:public BApplication{
 public:
  SSProcessRefsLooper *RefsLooper;
  SSApp();
  void MessageReceived(BMessage *msg);
  void RefsReceived(BMessage *msg);
  bool QuitRequested(void);
};

class SSWindow:public BWindow{
 public:
  SSWindow(BRect rect);
  bool QuitRequested();
};

class SSLabel:public BStringView{
 public:
  SSLabel(BRect rect);
  void MessageReceived(BMessage *msg);
};

vint32 CanQuit=1,IsClosing=0;
SSLabel *label=NULL;
BDirectory dir("/tmp");
BEntry tmpEntry("/tmp/SuperStripTemp");
int programFile;
const char *argsStrip[5]={"/boot/develop/tools/gnupro/bin/strip","","-o/tmp/SuperStripTemp","--strip-all","--discard-all"};
const char *argsCopyRes[3]={"/boot/beos/bin/copyres","","/tmp/SuperStripTemp"};
const char *argsCopyAttr[3]={"/boot/beos/bin/copyattr","","/tmp/SuperStripTemp"};
const char *argsCopy[3]={"/boot/beos/bin/cp","/tmp/SuperStripTemp",""};
const char *const attrName="mainWindowRect";
const char *const AppSignature="application/x-vnd.PFZ-SuperStrip";

int main(int argc,char *argv[]){
BEntry *entry;
BPath *path;
char *name;
SSApp *app;
 entry=new BEntry(argv[0],true);
 path=new BPath;
 entry->GetPath(path);
 delete entry;
 name=(char *)path->Path();
 programFile=open(name,O_RDWR|O_BINARY);
 // open the program file. This is used to read and write window position.
 delete path;
 app=new SSApp;
 app->Run();
 delete app;
 close(programFile);
 return 0;
}

// Here is the functions that strip the files. It is only a batch that will
// strip the file to /tmp/SuperStripTemp
// copyres from the original file to the /tmp/SuperStripTemp
// copyattr from the original file to the /tmp/SuperStripTemp
// and then copy the /tmp/SuperStripTemp over the original file.
// It only goes from one step to another if it doesn't detect any error on the previous
// step.
// Then it delete the /tmp/SuperStripTemp
// and if there is another file, do everithing again.
void SSProcessRefsLooper::MessageReceived(BMessage *msg){
 switch (msg->what){
  case B_REFS_RECEIVED:
  case B_SIMPLE_DATA:
   {
   entry_ref ref;
   BEntry entry;
   BPath path;
   int32 i;
   thread_id id;
   status_t retVal;
   char *name;
   int32 Closing=0;
    atomic_add(&CanQuit,-1);
    tmpEntry.Remove();
    for (i=0;msg->FindRef("refs",i,&ref)==B_OK;i++){
     Closing=atomic_or(&IsClosing,0);
     if (Closing) break;
     entry.SetTo(&ref,true);
     entry.GetPath(&path);
     name=(char *)path.Path();
     label->LockLooper();
     label->SetText(name);
     label->UnlockLooper();
     argsStrip[1]=name;
     argsCopyRes[1]=name;
     argsCopyAttr[1]=name;
     argsCopy[2]=name;
     id=load_image(5,(const char **)argsStrip,(const char **)environ);
     if (id>=0){
      wait_for_thread(id,&retVal);
      if (dir.Contains("SuperStripTemp",B_FILE_NODE)){
       id=load_image(3,(const char **)argsCopyRes,(const char **)environ);
       if ((id>=0)&&(retVal==0)){
        wait_for_thread(id,&retVal);
        id=load_image(3,(const char **)argsCopyAttr,(const char **)environ);
        if ((id>=0)&&(retVal==0)){
         wait_for_thread(id,&retVal);
         id=load_image(3,(const char **)argsCopy,(const char **)environ);
         if ((id>=0)&&(retVal==0)){
          wait_for_thread(id,&retVal);
         }
        }
       }
       tmpEntry.Remove();
      }
     }
    }
    label->LockLooper();
    label->SetText("Drop a file");
    label->UnlockLooper();
    atomic_add(&CanQuit,1);
    if (Closing) be_app->PostMessage(B_QUIT_REQUESTED);
   }
   break;
  default:
   BLooper::MessageReceived(msg);
   break;
 }
}

SSApp::SSApp():BApplication(AppSignature){
 try{
 BScreen scr;
 BRect r,scrRect=scr.Frame();
  if(fs_read_attr(programFile,attrName,B_RECT_TYPE,0,&r,sizeof(BRect))==sizeof(BRect)){
   if (r.right<0) r.right=0;
   if (r.bottom<0) r.bottom=0;
   if (r.left>scrRect.right) r.OffsetTo(scrRect.right,r.top);
   if (r.top>scrRect.bottom) r.OffsetTo(r.left,scrRect.bottom);
  } else {
   r=BRect(150,50,450,80);
  }
  RefsLooper=new SSProcessRefsLooper;
  RefsLooper->Run();
  new SSWindow(r);
 } catch(...){
  ShowMessage("Error initializing");
  PostMessage(B_QUIT_REQUESTED);
 }
}

void SSApp::MessageReceived(BMessage *msg){
 switch(msg->what){
  case B_SIMPLE_DATA:
   RefsLooper->PostMessage(msg);
   break;
  default:
   BApplication::MessageReceived(msg);
   break;
 }
}

void SSApp::RefsReceived(BMessage *msg){
 RefsLooper->PostMessage(msg);
}

bool SSApp::QuitRequested(void){
 atomic_or(&IsClosing,1);
int32 canQuit;
 canQuit=atomic_or(&CanQuit,0);
 if (!canQuit) return false;
 return BApplication::QuitRequested();
}

SSWindow::SSWindow(BRect rect):BWindow(rect,"SuperStrip",B_TITLED_WINDOW,B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS){
 AddChild(label=new SSLabel(Bounds()));
 label->LockLooper();
 label->SetAlignment(B_ALIGN_CENTER);
 label->UnlockLooper();
 Show();
}

bool SSWindow::QuitRequested(){
 atomic_or(&IsClosing,1);
int32 canQuit;
 canQuit=atomic_or(&CanQuit,0);
 if (canQuit){
 BRect MainWindowRect;
  MainWindowRect=Frame();
  fs_write_attr(programFile,attrName,B_RECT_TYPE,0,&MainWindowRect,sizeof(BRect));
  // This writes the window position, if somebody knows a better way to do this, please
  // tell me.
  be_app->PostMessage(B_QUIT_REQUESTED);
  return BWindow::QuitRequested();
 }
 return false;
}

SSLabel::SSLabel(BRect rect):BStringView(rect,"labelFile","Drop a file",B_FOLLOW_ALL_SIDES,B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW){
}

void SSLabel::MessageReceived(BMessage *msg){
 if (msg->what==B_SIMPLE_DATA){
  ((SSApp *)be_app)->RefsLooper->PostMessage(msg);
 } else {
  BStringView::MessageReceived(msg);
 }
}